#include "QtKinectExample.h"
#include "ui_QtKinectExample.h"

#include <QtKinect.h>
#include <QDebug>

#define PREVIEW_X 640
#define PREVIEW_Y 480


QtKinectExample::QtKinectExample(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::QtKinectExample),
    m_handCoordinatesLabel(0),
    m_handSquare(0)
{
    ui->setupUi(this);
    m_kinect =  new QtKinect(this);

    qRegisterMetaType< XnPoint3D >("XnPoint3D");
    qRegisterMetaType< XnUInt32>("XnUInt32");

    connect( m_kinect, SIGNAL(message(QString)), ui->outputText, SLOT(append(QString)));
    connect( m_kinect, SIGNAL(pointGenerated(XnPoint3D, XnUInt32)), this, SLOT(setPointOnView(XnPoint3D, XnUInt32)));
    connect( m_kinect, SIGNAL(pushDetected()), this, SLOT(kinectPushGestureDetected()));

    connect(ui->led_blinkGreen, SIGNAL(clicked()), this, SLOT(newLEDState()));
    connect(ui->led_green, SIGNAL(clicked()), this, SLOT(newLEDState()));
    connect(ui->led_red, SIGNAL(clicked()), this, SLOT(newLEDState()));
    connect(ui->led_off, SIGNAL(clicked()), this, SLOT(newLEDState()));
    connect(ui->led_yellowRed, SIGNAL(clicked()), this, SLOT(newLEDState()));
    connect(ui->led_yellow, SIGNAL(clicked()), this, SLOT(newLEDState()));

    connect (ui->slide_Tilt, SIGNAL(valueChanged(int)), this, SLOT(onSlideTiltValueChanged(int)));

    connect(ui->bt_clearPreview, SIGNAL(clicked()), this, SLOT(onBtClearPreviewClicked()));
    m_kinect->initDevice();
    m_scene = new QGraphicsScene(0, 0, PREVIEW_X, PREVIEW_Y, ui->previewScene);
    m_scene->setBackgroundBrush(QBrush(Qt::lightGray));
    ui->previewScene->setScene(m_scene);

    onBtClearPreviewClicked();
}

QtKinectExample::~QtKinectExample()
{
    delete m_kinect;
    delete ui;
}

void QtKinectExample::newLEDState()
{
    if( ui->led_off->isChecked())
        m_kinect->setLed(LED_OFF);

    else if(ui->led_green->isChecked())
        m_kinect->setLed(LED_GREEN);

    else if(ui->led_red->isChecked())
        m_kinect->setLed(LED_RED);

    else if(ui->led_yellow->isChecked())
        m_kinect->setLed(LED_YELLOW);

    else if(ui->led_blinkGreen->isChecked())
        m_kinect->setLed(LED_BLINK_GREEN);

    else if(ui->led_yellowRed->isChecked())
        m_kinect->setLed(LED_BLINK_RED_YELLOW);
}

void QtKinectExample::onSlideTiltValueChanged(int value)
{
    m_kinect->setTilt(value);
}

void QtKinectExample::setPointOnView(XnPoint3D handPoint, XnUInt32 handID)
{
    //adapting the point to match the scene coordinates
    QPointF adaptedPoint(handPoint.X + PREVIEW_X/2, -handPoint.Y + PREVIEW_Y/2);

        m_handSquare->setX(adaptedPoint.x());
        m_handSquare->setY(adaptedPoint.y());

    QString corrdinateStr = QString("<font color=\"#333333\">(%1, %2)</font>").arg(adaptedPoint.x()).arg(adaptedPoint.y());
    if(!m_scene->items().contains(m_handCoordinatesLabel)) {
        m_handCoordinatesLabel = new QGraphicsTextItem();
        m_scene->addItem(m_handCoordinatesLabel);
    }
    m_handCoordinatesLabel->setHtml(corrdinateStr);
    m_handCoordinatesLabel->setX(adaptedPoint.x()+17); // offset the text
    m_handCoordinatesLabel->setY(adaptedPoint.y());
    m_handCoordinatesLabel->setZValue(256); // make sure the text is above
    m_handCoordinatesLabel->setFont(QFont("Times", 9, QFont::Bold));
}

void QtKinectExample::kinectPushGestureDetected()
{
    m_scene->addEllipse(m_handSquare->x(), m_handSquare->y(), 40,40, QPen(), QBrush(QColor(qrand()%200, qrand()%200, qrand()%200)));
}

void QtKinectExample::onBtClearPreviewClicked()
{
    m_scene->clear();

    m_handSquare = new QGraphicsRectItem(0,0,15,15);
    m_handSquare->setBrush(Qt::white);
    m_handSquare->setZValue(256); // make sure the hand is above
    m_scene->addItem(m_handSquare);

}
